
############################
# GDI SPECIFIC CODE (Michael Schwartz)

# GDI is a dll for saving canvases to file or clipboard.
# See: http://www.du.edu/%7Emschwart/tcl-tk.htm

# This file provides interface code to that package.
# The code in this file written by Mick O'Donnell
# Define the versions for each tcl version


proc myPrintCanvas.gdi {cnv} {
    gdi-require {gdi printer}
    printer::print_widget $cnv
}


# GDI mySaveWgts
proc mySaveWgts.gdi {cnv wgts file} {

    # Create a pointer to the wmfdc
    set wmfdc [copy-wgts-to-wmfdc $cnv $wgts]

    # Copy to the file
    wmf copy $wmfdc -file [file nativename $file]

    # Delete the data
    wmf delete $wmfdc
}



proc myCaptureCanv.gdi {cnv} {

    set wgts [$cnv find all]
    myCaptureWgts.gdi $cnv $wgts
}


# GDI myCaptureWgts
proc myCaptureWgts.gdi {cnv wgts} {
        
    # Create a pointer to the wmfdc
    set wmfdc [copy-wgts-to-wmfdc $cnv $wgts]

    # Copy to the clipboard
    wmf copy $wmfdc

    # Delete the data
    wmf delete $wmfdc
}


proc copy-wgts-to-wmfdc {cnv wgts} {
    # Create the wmf handle
    set bbox [$cnv bbox $wgts]
    set hDCname [wmf open]
    
    # Draw the widgets to the wmf
    gdi-copy-widgets $hDCname $cnv $wgts
    
    # close the 
    return [wmf close $hDCname]
}

proc gdi-copy-widgets {hDCname cnv wgts} {
    # 1. Capture the bbox of these widgets to find the xy offset
    set bbox [eval [concat "$cnv bbox" $wgts]]
    set xoffset [lindex $bbox 0]
    set yoffset [lindex $bbox 1]

    foreach wgt $wgts {
	set coords1 [$cnv coords $wgt]
	# Correct the coords for offset
	set coords {}
	set len [llength $coords1]
	set i 0
	while { $i < $len} {
	    lappend coords [expr [lindex $coords1 $i] - $xoffset]
	    incr i
	    lappend coords [expr [lindex $coords1 $i] - $yoffset]
	    incr i
	}
	set color [$cnv itemcget $wgt -fill]
	
	switch --  [$cnv type $wgt] {
	    text {
		set x [lindex $coords 0]
		set y [lindex $coords 1]

		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}

		set txt [$cnv itemcget $wgt -text]
		set fontdetails [font actual [$cnv itemcget $wgt -font]]
		set family [getarg  -family $fontdetails]

		set size [getarg  -size $fontdetails]
		set slant [getarg  -slant $fontdetails]
		if { $slant != "italic"} {
		    set slant ""
		}
		set weight [getarg  -weight $fontdetails]
		set font "\{$family\} $size $slant $weight"
		set anchor [$cnv itemcget $wgt -anchor]
		set bbox [$cnv bbox $wgt]
		# Add 2 because some seems to be lost in the translation
		set wdth [expr [lindex $bbox 2] - [lindex $bbox 0] +2]
		set hgth [expr [lindex $bbox 3] - [lindex $bbox 1] ]
		# $cnv create line $x $y [expr $x + $wdth] $y -fill red
		# $cnv create line $x [expr $y + $hgth] [expr $x + $wdth] [expr $y + $hgth] -fill red
		gdi text $hDCname $x $y -text $txt -fill $color -font $font -anchor $anchor\
			-width $wdth -unicode

#		gdi line  $hDCname $x $y [expr $x + $wdth] $y -fill red
	#	gdi line  $hDCname $x [expr $y + $hgth] [expr $x + $wdth] [expr $y + $hgth] -fill red
	    }
	    
	    line {
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}

		set joinstyle [$cnv itemcget $wgt -joinstyle]
		set smooth [$cnv itemcget $wgt -smooth]
		set splinesteps [$cnv itemcget $wgt -splinesteps]
		set arrow [$cnv itemcget $wgt -arrow]
		
		switch -- $smooth {
		    0 {set smooth false}
		    1 -
		    "bezier" {set smooth true}
		}
		
		# Smoothed lines come out terrible, better unsmoothed
		# set smooth false

		eval "gdi line $hDCname $coords -fill $color -joinstyle $joinstyle\
			-smooth $smooth -splinesteps $splinesteps -arrow $arrow"
	    }
	    
	    arc {
		#gdi arc hdc x1 y1 x2 y2 -extent degrees -fill color -outline color 
 	        #        -outlinestipple bitmap -start degrees -stipple bitmap 
 		#        -style [pieslice|chord|arc] -width linewid
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}
		eval "gdi arc $hDCname $coords $attribs"
	    }
	    
	    rectangle {
		set outline [$cnv itemcget $wgt -outline]
		set fill [$cnv itemcget $wgt -fill]
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}
		eval "gdi rectangle $hDCname $coords -outline $outline -fill $fill"
	    }

            oval {
                set outline [$cnv itemcget $wgt -outline]
		set fill [$cnv itemcget $wgt -fill]
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}
		eval "gdi oval $hDCname $coords -outline $outline -fill $fill"
	    }
            
	    default {
		#puts "Type: [$cnv type $wgt] not catered to"
	    }
	}
    }
}

proc getarg {key list} {
# Returns the value in list immediately following key
    for {set i 0} {$i < [llength $list]} {incr i 2} {
	if { [lindex $list $i] == $key } {
	    return [lindex $list [expr $i + 1]]
	}
    }
    return {}
}
